/*
 * Braille Utils (C) 2010-2011 Daisy Consortium
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.daisy.braille.utils.api.paper;

/**
 *
 * @author Bert Frees
 */
public class Area implements Dimensions {

	private final double width, height, offsetX, offsetY;

	/**
	 * Creates a new area with the specified parameters
	 * @param width the width, in mm
	 * @param height the height, in mm
	 * @param offsetX the x-offset, in mm
	 * @param offsetY the y-offset, in mm
	 */
	public Area(double width,
			double height,
			double offsetX,
			double offsetY) {

		this.width = width;
		this.height = height;
		this.offsetX = offsetX;
		this.offsetY = offsetY;
	}

	@Override
	public double getWidth() {
		return width;
	}

	@Override
	public double getHeight() {
		return height;
	}

	/**
	 * Gets the x-offset.
	 * @return returns the x-offset, in mm
	 */
	public double getOffsetX() {
		return offsetX;
	}

	/**
	 * Gets the y-offset.
	 * @return returns the y-offset, in mm
	 */
	public double getOffsetY() {
		return offsetY;
	}
}
